import { TimeConverter } from '@modules/ad/infrastructure/time-converter';

describe('Time Converter', () => {
  it('should be defined', () => {
    const timeConverter: TimeConverter = new TimeConverter();
    expect(timeConverter).toBeDefined();
  });

  describe('localStringTimeToUtcStringTime', () => {
    it('should convert a paris time to utc time', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisTime = '08:00';
      const utcDatetime = timeConverter.localStringTimeToUtcStringTime(
        parisTime,
        'Europe/Paris',
      );
      expect(utcDatetime).toBe('07:00');
    });
    it('should throw an error if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const fooBarTime = '08:00';
      expect(() => {
        timeConverter.localStringTimeToUtcStringTime(fooBarTime, 'Foo/Bar');
      }).toThrow();
    });
  });

  describe('utcStringTimeToLocalStringTime', () => {
    it('should convert a utc time to a paris time', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcTime = '07:00';
      const parisTime = timeConverter.utcStringTimeToLocalStringTime(
        utcTime,
        'Europe/Paris',
      );
      expect(parisTime).toBe('08:00');
    });
    it('should throw an error if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcTime = '27:00';
      expect(() => {
        timeConverter.utcStringTimeToLocalStringTime(utcTime, 'Europe/Paris');
      }).toThrow();
    });
    it('should throw an error if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcTime = '07:00';
      expect(() => {
        timeConverter.utcStringTimeToLocalStringTime(utcTime, 'Foo/Bar');
      }).toThrow();
    });
  });

  describe('localStringDateTimeToUtcDate', () => {
    it('should convert a summer paris date and time to a utc date with dst', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
        true,
      );
      expect(utcDate.toISOString()).toBe('2023-06-22T10:00:00.000Z');
    });
    it('should convert a winter paris date and time to a utc date with dst', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-02-02';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
        true,
      );
      expect(utcDate.toISOString()).toBe('2023-02-02T11:00:00.000Z');
    });
    it('should convert a summer paris date and time to a utc date without dst', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        parisDate,
        parisTime,
        'Europe/Paris',
      );
      expect(utcDate.toISOString()).toBe('2023-06-22T11:00:00.000Z');
    });
    it('should convert a tonga date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const tongaDate = '2023-02-02';
      const tongaTime = '12:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        tongaDate,
        tongaTime,
        'Pacific/Tongatapu',
      );
      expect(utcDate.toISOString()).toBe('2023-02-01T23:00:00.000Z');
    });
    it('should convert a papeete date and time to a utc date', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const papeeteDate = '2023-02-02';
      const papeeteTime = '15:00';
      const utcDate = timeConverter.localStringDateTimeToUtcDate(
        papeeteDate,
        papeeteTime,
        'Pacific/Tahiti',
      );
      expect(utcDate.toISOString()).toBe('2023-02-03T01:00:00.000Z');
    });
    it('should throw an error if date is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-32';
      const parisTime = '08:00';
      expect(() => {
        timeConverter.localStringDateTimeToUtcDate(
          parisDate,
          parisTime,
          'Europe/Paris',
        );
      }).toThrow();
    });
    it('should throw an error if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '28:00';
      expect(() => {
        timeConverter.localStringDateTimeToUtcDate(
          parisDate,
          parisTime,
          'Europe/Paris',
        );
      }).toThrow();
    });
    it('should throw an error if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const parisDate = '2023-06-22';
      const parisTime = '12:00';
      expect(() => {
        timeConverter.localStringDateTimeToUtcDate(
          parisDate,
          parisTime,
          'Foo/Bar',
        );
      }).toThrow();
    });
  });

  describe('utcStringDateTimeToLocalIsoString', () => {
    it('should convert a utc string date and time to a summer paris date isostring with dst', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '10:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
        true,
      );
      expect(localIsoString).toBe('2023-06-22T12:00:00.000+02:00');
    });
    it('should convert a utc string date and time to a winter paris date isostring with dst', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-02-02';
      const utcTime = '10:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
        true,
      );
      expect(localIsoString).toBe('2023-02-02T11:00:00.000+01:00');
    });
    it('should convert a utc string date and time to a summer paris date isostring', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '10:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Europe/Paris',
      );
      expect(localIsoString).toBe('2023-06-22T11:00:00.000+01:00');
    });
    it('should convert a utc date to a tonga date isostring', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-02-01';
      const utcTime = '23:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Pacific/Tongatapu',
      );
      expect(localIsoString).toBe('2023-02-02T12:00:00.000+13:00');
    });
    it('should convert a utc date to a papeete date isostring', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-02-03';
      const utcTime = '01:00';
      const localIsoString = timeConverter.utcStringDateTimeToLocalIsoString(
        utcDate,
        utcTime,
        'Pacific/Tahiti',
      );
      expect(localIsoString).toBe('2023-02-02T15:00:00.000-10:00');
    });
    it('should throw an error if date is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-32';
      const utcTime = '07:00';
      expect(() => {
        timeConverter.utcStringDateTimeToLocalIsoString(
          utcDate,
          utcTime,
          'Europe/Paris',
        );
      }).toThrow();
    });
    it('should throw an error if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '27:00';
      expect(() => {
        timeConverter.utcStringDateTimeToLocalIsoString(
          utcDate,
          utcTime,
          'Europe/Paris',
        );
      }).toThrow();
    });
    it('should throw an error if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      const utcDate = '2023-06-22';
      const utcTime = '07:00';
      expect(() => {
        timeConverter.utcStringDateTimeToLocalIsoString(
          utcDate,
          utcTime,
          'Foo/Bar',
        );
      }).toThrow();
    });
  });

  describe('utcUnixEpochDayFromTime', () => {
    it('should get the utc day of paris at 12:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('12:00', 'Europe/Paris'),
      ).toBe(4);
    });
    it('should get the utc day of paris at 00:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('00:00', 'Europe/Paris'),
      ).toBe(3);
    });
    it('should get the utc day of papeete at 16:00', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.utcUnixEpochDayFromTime('16:00', 'Pacific/Tahiti'),
      ).toBe(5);
    });
    it('should throw an error if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(() => {
        timeConverter.utcUnixEpochDayFromTime('28:00', 'Europe/Paris');
      }).toThrow();
    });
    it('should throw an error if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(() => {
        timeConverter.utcUnixEpochDayFromTime('12:00', 'Foo/Bar');
      }).toThrow();
    });
  });

  describe('localUnixEpochDayFromTime', () => {
    it('should get the day of paris at 12:00 utc', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('12:00', 'Europe/Paris'),
      ).toBe(4);
    });
    it('should get the day of paris at 23:00 utc', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('23:00', 'Europe/Paris'),
      ).toBe(5);
    });
    it('should get the day of papeete at 05:00 utc', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(
        timeConverter.localUnixEpochDayFromTime('05:00', 'Pacific/Tahiti'),
      ).toBe(3);
    });
    it('should throw an error if time is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(() => {
        timeConverter.localUnixEpochDayFromTime('28:00', 'Europe/Paris');
      }).toThrow();
    });
    it('should throw an error if timezone is invalid', () => {
      const timeConverter: TimeConverter = new TimeConverter();
      expect(() => {
        timeConverter.localUnixEpochDayFromTime('12:00', 'Foo/Bar');
      }).toThrow();
    });
  });
});
